// Auxiliary functions
// Tactics - 24/04/2006

#include "stdlib.h"
#include "string.h"
#include "gccore.h"
#include "ogcsys.h"

#include "ogc/card.h"

#include "viperusb.h"
#include "umcproto.h"

static char workarea[CARD_WORKAREA] ATTRIBUTE_ALIGN(32);

void card_removed(s32 chn,s32 result)
{
 printf ("Card %d removed!\n", chn+1);
 CARD_Unmount(chn);
}

u32 CalculateCheckSum (u8 *buffer, u32 tam)
{
 u32 temp, i;

 temp = 0;
 for (i = 0; i < tam; i++) 
  temp += buffer[i];

 return temp;
}

void EraseFile ()
{
 int err;
 card_dir temp;

 USBReceiveBufferB ((unsigned char *) &temp, sizeof (card_dir));

 printf ("%s (%s-%s)... ", temp.filename, temp.gamecode, temp.company);

 if ( ((err = CARD_ReInit (temp.gamecode, temp.company)) < 0) ||
      ((err = CARD_Mount (temp.chn, workarea, card_removed)) < 0))
  {
   USBSendByteB (CMD_ERR);
   return;
  }

 err = CARD_Delete (temp.chn, temp.filename);
 if (err < 0)
   {
    USBSendByteB (CMD_ERR);
    printf ("\n   Error trying to erase file %s!\n", temp.filename);
   }
  else
    USBSendByteB (CMD_ACK);

 CARD_Unmount (temp.chn);
}

void Upload ()
{
 int i, j;
 int channel, err;
 unsigned char channelbyte, cmd;
 char gamecode[5], company[3], filename[32+1];
 u32 checksum1, checksum2, sectorsize;

 card_file file;
 struct card_direntry status;
 char *CardBuffer;

 USBReceiveBufferB (&channelbyte, 1);
 if ((channelbyte != 0) && (channelbyte != 1))
  {
   printf ("\n   Error receiving memory card number!\n");
   USBSendByteB (CMD_ERR);
   return;
  }
 channel = channelbyte;

 if ( ((err = CARD_ReInit ("    ", "  ")) < 0) ||
      ((err = CARD_Mount (channel, workarea, card_removed)) < 0))
  {
   printf ("\n   Error mounting memory card %d!\n", channel+1);
   USBSendByteB (CMD_ERR);
   return;
  }

 CARD_GetSectorSize (channel, &sectorsize);
 CardBuffer = (char *) memalign(32, sectorsize);
 USBSendByteB (CMD_ACK);
 USBSendBufferB ((unsigned char *) &sectorsize, 4);

 USBReceiveBufferB ((unsigned char *) &status, sizeof (struct card_direntry));
 company[2] = 0;
 gamecode[4] = 0;
 filename[32] = 0;
 strncpy (company, status.company, 2);
 strncpy (gamecode, status.gamecode, 4);
 strncpy (filename, status.filename, 32);

 printf ("%s (%s-%s)...\n   ", filename, gamecode, company);

 if ( ((err = CARD_ReInit (gamecode, company)) < 0) ||
      ((err = CARD_Mount (channel, workarea, card_removed)) < 0) ||
      ((err = CARD_Create (channel, filename, status.length * sectorsize, &file)) < 0))
  {
   printf ("Error mounting memory card and creating new file!\n");
   if (err == CARD_ERROR_EXIST)
     printf ("   (file probably already exists)\n");
   USBSendByteB ((unsigned char) err);
   CARD_Unmount (channel);
   free (CardBuffer);
   return;
  }
 
 // Update the incoming directory entry with the first block of the
 // just created file
 status.block = file.iblock;

 if ((err =  CARD_SetRawStatus (channel, file.filenum, &status)) < 0)
  {
   printf ("Error setting directory entry details!\n");
   USBSendByteB ((unsigned char) err);
   CARD_Unmount (channel);
   free (CardBuffer);
   return;
  }

 USBSendByteB (CMD_ACK);

 printf ("Receiving %d blocks: ", status.length);
 j = status.length;
 for (i = 0; i < j; )
  {
   USBReceiveBufferB (CardBuffer, sectorsize);
   checksum1 = CalculateCheckSum (CardBuffer, sectorsize);
   USBReceiveBufferB ((unsigned char *) &checksum2, 4);
   if (checksum1 != checksum2)
    {
     printf ("*");
     USBSendByteB (CMD_ERR);
     continue;
    }

   err = CARD_Write (&file, CardBuffer, sectorsize, i * sectorsize);
   if (err < 0)
     {
      printf ("*");
      USBSendByteB (CMD_ERR);
      continue; // Rewite the error handling code
     }
    else
     {
      printf (".");
     }

   USBSendByteNonB (CMD_ACK);
   i++;
  }

 USBReceiveBufferB (&cmd, 1);
 if (cmd != CMD_END)
   printf ("   File size error. Erase and start over.\n");
 CARD_Close (&file);
 CARD_Unmount (channel);
 free (CardBuffer);
}

void CheckDump ()
{
 int i, j;
 int channel, err;
 unsigned char *p1, *p2;
 unsigned char channelbyte, cmd;
 char gamecode[5], company[3], filename[32+1];
 u32 checksum1, checksum2, sectorsize;

 card_file file;
 struct card_direntry status, status2;
 char *CardBuffer, *CardBuffer2;

 USBReceiveBufferB (&channelbyte, 1);
 if ((channelbyte != 0) && (channelbyte != 1))
  {
   USBSendByteB (CMD_ERR);
   printf ("Error receiving memory card number!\n");
   return;
  }
 channel = channelbyte;

 if ( ((err = CARD_ReInit ("    ", "  ")) < 0) ||
      ((err = CARD_Mount (channel, workarea, card_removed)) < 0))
  {
   printf ("Error mounting memory card %d!\n", channel+1);
   USBSendByteB (CMD_ERR);
   return;
  }

 CARD_GetSectorSize (channel, &sectorsize);
 CardBuffer = (char *) memalign(32, sectorsize);
 CardBuffer2 = (char *) memalign(32, sectorsize);
 USBSendByteB (CMD_ACK);
 USBSendBufferB ((unsigned char *) &sectorsize, 4);

 USBReceiveBufferB ((unsigned char *) &status, sizeof (struct card_direntry));
 company[2] = 0;
 gamecode[4] = 0;
 filename[32] = 0;
 strncpy (company, status.company, 2);
 strncpy (gamecode, status.gamecode, 4);
 strncpy (filename, status.filename, 32);

 printf (" %s\n   Company : %s\n   Gamecode : %s\n   ", filename, company, gamecode);

 if ( ((err = CARD_ReInit (gamecode, company)) < 0) ||
      ((err = CARD_Mount (channel, workarea, card_removed)) < 0) ||
      ((err = CARD_Open(channel, filename, &file)) < 0))
  {
   printf ("Error mounting memory card!\n");
   USBSendByteB ((unsigned char) err);
   CARD_Unmount (channel);
   free (CardBuffer);
   free (CardBuffer2);
   return;
  }

 err = CARD_GetRawStatus(channel, file.filenum, &status2);
 if (err < 0)
  {
   printf ("Error reading directory entry!\n");
   USBSendByteB ((unsigned char) err);
   CARD_Unmount (channel);
   free (CardBuffer);
   free (CardBuffer2);
   return;
  }

 printf ("Directory entry different offsets: (0x36 and 0x37 are ok)\n    ");
 p1 = (unsigned char *) &status;
 p2 = (unsigned char *) &status2;
 for (i = 0; i < sizeof (struct card_direntry); i++)
  {
   if (p1[i] != p2[i])
    printf ("0x%02X ", i);
  }

 printf ("Finished\n   ");
 USBSendByteB (CMD_ACK);
 
 j = status.length;
 printf ("Checking %d blocks: ", j);
 for (i = 0; i < j; i++)
  {
   USBReceiveBufferB (CardBuffer, sectorsize);
   checksum1 = CalculateCheckSum (CardBuffer, sectorsize);
   USBReceiveBufferB ((unsigned char *) &checksum2, 4);
   if (checksum1 != checksum2)
    {
     USBSendByteB (CMD_ERR);
     continue;
    }

   err = CARD_Read(&file, CardBuffer2, sectorsize, i * sectorsize);
   if (err < 0) printf ("\n   Error reading file!\n   ");

   if (memcmp (CardBuffer, CardBuffer2, sectorsize) != 0)
     printf ("*");
    else
     printf (".");

   USBSendByteNonB (CMD_ACK);
  }

 USBReceiveBufferB (&cmd, 1);
 if (cmd != CMD_END)
   printf ("\n   File size error! ");

 CARD_Close (&file);
 CARD_Unmount (channel);
 free (CardBuffer);
 free (CardBuffer2);
}

void DumpFile ()
{
 int err;
 u8 resp;
 u32 i, checksum, total, sectorsize;

 card_dir temp;
 card_file file;
 struct card_direntry status;
 char *CardBuffer;

 USBReceiveBufferB ((unsigned char *) &temp, sizeof (card_dir));
 printf (" %s (%s-%s)\n   ", temp.filename, temp.gamecode, temp.company);

 if ( ((err = CARD_ReInit (temp.gamecode, temp.company)) < 0) ||
      ((err = CARD_Mount (temp.chn, workarea, card_removed)) < 0))
  {
   printf ("Error mounting memory card!\n");
   USBSendByteB (CMD_ERR);
   return;
  }

 CARD_GetSectorSize(temp.chn, &sectorsize);
 err = CARD_Open(temp.chn, temp.filename, &file);
 if ((err < 0) || (sectorsize < 1))
  {
   printf ("Error opening file!\n");
   CARD_Unmount (temp.chn);
   USBSendByteB ((unsigned char) err);
   return;
  }

 err = CARD_GetRawStatus(temp.chn, file.filenum, &status);
 if (err < 0)
  {
   printf ("Error reading directory entry!\n");
   CARD_Close (&file);
   CARD_Unmount (temp.chn);
   USBSendByteB ((unsigned char) err);
   return;
  }
   
 USBSendByteB (CMD_ACK);
 USBSendBufferB ((unsigned char *) &status, sizeof (struct card_direntry));
 USBSendBufferB ((unsigned char *) &file.len, 4);
 USBSendBufferB ((unsigned char *) &sectorsize, 4);
 total = file.len / sectorsize;

 printf ("Sending %d blocks: ", total);
 CardBuffer = (char *) memalign(32, sectorsize);
 for (i = 0; i < total; )
  {
   err = CARD_Read(&file, CardBuffer, sectorsize, i * sectorsize);
   if (err < 0)
    {
     printf ("\n   Error reading block %d!", i + 1);
     USBSendByteB (CMD_ERR);
     break;
    }
   USBSendByteB (CMD_ACK);
   USBSendBufferB ((unsigned char *) &sectorsize, 4);
   USBSendBufferB ((unsigned char *) CardBuffer, sectorsize);
   checksum = CalculateCheckSum (CardBuffer, sectorsize);
   USBSendBufferB ((unsigned char *) &checksum, 4);    

   USBReceiveByteB (&resp);
   if (resp == CMD_ACK)
     {
      printf (".");
      i++;
     }
    else
     printf ("*");
  }

 if (i == total)
   USBSendByteB (CMD_END);

 CARD_Close (&file);
 CARD_Unmount (temp.chn);
 free (CardBuffer);
}

void ListMemCard (int channel)
{
 int err = 0;
 card_dir temp;

 memset (&temp, 0, sizeof (card_dir));

 if ( ((err = CARD_ReInit ("    ", "  ")) >= 0) &&
      ((err = CARD_Mount (channel, workarea, card_removed)) >= 0) &&
      ((err = CARD_FindFirst (channel, &temp, true)) >= 0))
  {
   do
    { 
     USBSendByteB (CMD_ACK);
     USBSendBufferB ((unsigned char*) &temp, sizeof(card_dir));
    } while (CARD_FindNext (&temp) >= 0);
   USBSendByteB (CMD_END); 
  }
 else
  {
   if (err == CARD_ERROR_NOFILE)
     USBSendByteB (CMD_END);
    else
     USBSendByteB (CMD_ERR);
  }

 CARD_Unmount (channel);
}
