/****************************************************************************
 *  USBMemCardGC - A Memory card dumper for the Viper GC Extreme
 *
 *  This source file is based on the GenPlus source code.
 *  Special thanks to the SoftDev team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ***************************************************************************/

#include "stdlib.h"
#include "string.h"
#include "gccore.h"
#include "ogcsys.h"
#include "ogc/card.h"

#include "viperusb.h"
#include "umcproto.h"
#include "cmds.h"

// 2D Video
unsigned int *xfb = NULL; // Single buffered
GXRModeObj *vmode;		  // General video mode

// InitGCVideo
// This function MUST be called at startup.
void InitGCVideo ()
{
 // Before doing anything else under libogc, call VIDEO_Init
 VIDEO_Init ();

 // Detect and reset the video mode. This is always set to 60hz
 // whether your running PAL or NTSC
 switch (VIDEO_GetCurrentTvMode ())
  {
   case VI_NTSC:
     vmode = &TVNtsc480IntDf;
     break;

   case VI_PAL:
   case VI_MPAL:
     vmode = &TVMpal480IntDf;
     break;

   default:
     vmode = &TVNtsc480IntDf;
     break;
  }

#ifdef FORCE_PAL50
 vmode = &TVPal528IntDf;
#endif
 VIDEO_Configure (vmode);

 // Allocate the video buffers
 xfb = (unsigned int *) MEM_K0_TO_K1 (SYS_AllocateFramebuffer (vmode));

 // A console is always useful
 VIDEO_SetNextFramebuffer(xfb);
 VIDEO_SetBlack(FALSE);
 VIDEO_Flush();
 VIDEO_WaitVSync();
 if (vmode->viTVMode & VI_NON_INTERLACE) VIDEO_WaitVSync();
 console_init(xfb, 20, 64, vmode->fbWidth, vmode->xfbHeight, vmode->fbWidth*2);
 VIDEO_SetNextFramebuffer(xfb);
}

// Main
int main (int argc, char *argv[])
{
 char cmd;

 InitGCVideo ();
 USBFlushInputBuffer();
 printf ("USBMemCardGC 1.0 - by Tactics\nWaiting commands...\n");

 while (1)
  {
   USBReceiveByteB (&cmd);

   switch (cmd)
    {
     case CMD_LISTDIR_MEMCARD_1:
      printf ("   Listing Memory Card 1 contents... ");
      ListMemCard (CARD_SLOTA);
      break;

     case CMD_LISTDIR_MEMCARD_2:
      printf ("   Listing Memory Card 2 contents... ");
      ListMemCard (CARD_SLOTB);
      break;

     case CMD_ERASEFILE:
      printf ("   Erasing file ");
      EraseFile();
      break;

     case CMD_DUMPFILE:
      printf ("   Dumping file...");
      DumpFile ();
      break;

     case CMD_CHECKDUMP:
      printf ("   Checking file...");
      CheckDump();
      break;

     case CMD_UPLOAD:
      printf ("   Receiving file ");
      Upload ();
      break;
    }
   printf ("Done.\n");
  };

 return 0;
}
