// USBMemCardGC's commands

//--------------------
// Sends an array of card_dir structs to the PC (check libogc's card.h)
#define CMD_LISTDIR_MEMCARD_1  0x1   
#define CMD_LISTDIR_MEMCARD_2  0x2
/* PC : CMD_LISTDIR_MEMCARD_X
   GC : CMD_ACK
        card_dir
        CMD_ACK
        card_dir
        ...
        CMD_END
*/

//--------------------
// Dump a file from the GC's memory card to the PC
#define CMD_DUMPFILE           0x3
/* PC : CMD_DUMPFILE
        card_dir       (retrieved calling CMD_LISTDIR_MEMCARD_X - it contains the memory card number)
   GC : CMD_ACK
        card_direntry  (directory entry's 64 bytes, struct card_direntry - see libogc's card.c, or attached card.h)
        4 bytes -->    total file size, in bytes. (endianess converted by the PC)
        4 bytes -->    memory card's sector size. The files are transfered in blocks of this size.

 Then the GC will send blocks like these, and the PC will answer CMD_ACK every time.
   GC : CMD_ACK
        4 bytes -->    block size (equals sector size sent before - I thought this could be useful, but doesn't seem so)
        'block size' bytes
        4 bytes -->    a simple checksum
 After the last block, the GC will send a CMD_END and finish the transaction.
 If instead of a CMD_ACK, the PC answer a CMD_ERR, the GC will resend the last block sent.
*/

//--------------------
// Erases a file from the GC's memory card
#define CMD_ERASEFILE          0x4
/* PC : CMD_ERASEFILE
        card_dir       (see description above)
   GC : CMD_ACK or CMD_ERR
*/

//-----------------------
// Writes a file to the GC's memory card. There can't be a file in the MC with the same name
#define CMD_UPLOAD             0x5
// Checks if a file in the PC is the same on the memory card
#define CMD_CHECKDUMP          0x6
/*
   PC : CMD_UPLOAD or CMD_CHECKDUMP
        1 byte   -->  0x0 for memory card 1, 0x1 for memory card 2
   GC : CMD_ACK
        4 bytes  -->  memory card's sector size. The files are transfered in blocks of this size.
   PC : card_stat
   GC:  CMD_ACK
 Then the PC will send blocks like these, and then a CMD_END.
   PC:  'sector size' bytes
        4 bytes  -->  simple checksum
 The GC always answers a CMD_ACK, unless there is a checksum mismatch.
*/

#define CMD_END                0xFD
#define CMD_ACK                0xFE
#define CMD_ERR                0xFF

#define CARD_DIR_TRANSMIT_SIZE  52
#define CARD_STAT_TRANSMIT_SIZE 64
