#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>

#include "viperusb.h"
#include "umcproto.h"

struct Comando  // menu itens
{
 char *texto;
 void (*func) (void);
};

struct Lista
{
 char *arquivo;
 struct Lista *prox;  // next
};

// Extracted from libOGC (card.h)
#define CARD_FILENAMELEN			32
#define CARD_MAXICONS				8

typedef struct _card_dir { 
      int chn; 
      unsigned int fileno; 
      char filename[CARD_FILENAMELEN+1]; 
      char gamecode[5]; 
      char company[3];
      char showall;
      struct _card_dir *prox; // slight modification in the original structure to link them together
} card_dir; 

typedef struct _card_stat {
	unsigned char filename[CARD_FILENAMELEN];
	unsigned int len;
	unsigned int time;		//time since 1970 in seconds
	unsigned char gamecode[4];
	unsigned char company[2];
	unsigned char banner_fmt;
	unsigned int icon_addr;
	unsigned short icon_fmt;
	unsigned short icon_speed;
	unsigned int comment_addr;
	unsigned int offset_banner;
	unsigned int offset_banner_tlut;
	unsigned int offset_icon[CARD_MAXICONS];
	unsigned int offset_icon_tlut;
	unsigned int offset_data;
} card_stat;

struct card_direntry {
	unsigned char gamecode[4];
	unsigned char company[2];
	unsigned char pad_00;
	unsigned char bannerfmt;
	unsigned char filename[CARD_FILENAMELEN];
	unsigned int lastmodified;
	unsigned int iconaddr;
	unsigned short iconfmt;
	unsigned short iconspeed;
	unsigned char permission;
	unsigned char copytimes;
	unsigned short block;
	unsigned short length;
	unsigned short pad_01;
	unsigned int commentaddr;
} __attribute__((packed));

// Iterates through a card_dir list and calls func for each element
void PercorreLista (card_dir *p, BOOL (*func)(card_dir*))
{
 BOOL ret;
 card_dir *temp;

 // Kinda init...
 func (NULL);

 while (p != NULL)
  {
   temp = p->prox;
   ret = func(p);
   if (ret == FALSE) return;
   p = temp;
  }
}

// Frees an element in a card_dir list
BOOL LivraCardDir (card_dir *p)
{
 if (p == NULL)
   return FALSE;
  else
   free (p);

 return TRUE;
}

// Printfs an element in a card_dir list
BOOL ImprimeCardDir (card_dir *p)
{
 static int count = 1;

 if (p == NULL)
  {
   count = 1;
   return FALSE;
  }

 printf ("[%d] %32s (%4s - %2s)\n", count, p->filename, p->gamecode, p->company);
 count ++;
 return TRUE;
}

// Grabs an integer from the keyboard
BOOL PegaInt (int *valor)
{
 int ret;
 char *temp;
 char buffer[81];
 
 temp = fgets (buffer, 80, stdin);
 if (temp == NULL) return FALSE;
 if (strlen (temp) == 0) return FALSE;
 ret = atoi (buffer); 

 *valor = ret;
 return TRUE;
}

BOOL PegaMemCard (int *channel)
{
 int temp;

 printf ("Which memory card (1 ou 2)? ");
 if (PegaInt (&temp) == FALSE) return FALSE;

 if ((temp != 1) && (temp != 2))
  {
   printf ("There is no memory card %d!!\n", channel);
   return FALSE;
  }
 
 *channel = temp -1;
 return TRUE;
}

card_dir *CarregaCardDir (unsigned char cmd); // See below

// Retrieves the card_dir list from the GC, prints it
// and returns a pointer to the list
card_dir *ListaDirMemCard (int channel)
{
 card_dir *CD;

 if (channel == 0)
   CD = CarregaCardDir(CMD_LISTDIR_MEMCARD_1);
  else
   CD = CarregaCardDir(CMD_LISTDIR_MEMCARD_2);

 if (CD == NULL)
  {
   printf ("Error fetching memory card directory!\n");
   return NULL;
  }

 PercorreLista (CD, ImprimeCardDir);
 return CD;
}

// Asks the user which file in the memory card they want
card_dir *PegaArqDaLista (card_dir *lista)
{
 int arqnum, i;
 card_dir *temp; 

 printf ("Which file? ");
 if (PegaInt (&arqnum) == FALSE) return NULL;
 if (arqnum < 1) return NULL;

 temp = lista;
 for (i = 1; i < arqnum; i++)
  {
   temp = temp->prox;
   if (temp == NULL) return NULL;
  }

 return temp;
}

// Sends a command byte to the GC
void SendCommand (char cmd)
{
 DWORD len;
 ViperWrite (&cmd, 1, &len);
}

// Receive a 4 byte integer from the GC
DWORD GetDWORD ()
{ 
 DWORD ret, len;
 unsigned char *mask;

 mask = (unsigned char *) &ret;
 
 ViperRead (&mask[3], 1, &len);
 ViperRead (&mask[2], 1, &len);
 ViperRead (&mask[1], 1, &len);
 ViperRead (&mask[0], 1, &len);
 return ret;
}

// Sends a 4 byte integer to the GC
BOOL PutDWORD (DWORD n)
{ 
 DWORD len;
 unsigned char *p, tmp[4];

 p = (unsigned char *) &n;

 tmp[0] = p[3];
 tmp[1] = p[2];
 tmp[2] = p[1];
 tmp[3] = p[0];

 ViperWrite (tmp, 4, &len);
 if (len == 4)
   return TRUE;
  else
   return FALSE;
}

// Receives a command byte from the GC
BOOL GetCommand (unsigned char *cmd)
{
 DWORD len;
 unsigned char incoming;

 ViperRead (&incoming, 1, &len);
 if (len == 0) return FALSE;

 *cmd = incoming;
 return TRUE;
}

// Calculates a simple checksum. I should implement CRC32...
DWORD CalculaCheckSum (unsigned char *buffer, DWORD tam)
{
 DWORD temp, i;

 temp = 0;
 for (i = 0; i < tam; i++) 
  temp += buffer[i];

 return temp;
}

// Retrieves the list of files from the GC (card_dir list)
card_dir *CarregaCardDir (unsigned char cmd) 
{
 DWORD len;
 card_dir *CD, *temp, **pos;
 unsigned char resp;

 SendCommand (cmd);

 CD = NULL;
 pos = &CD;
 while (1)
  {

   if (GetCommand (&resp) == FALSE)
    {
     printf ("Error receiving GC's answer!\n");
     return NULL;
    }

   if (resp == CMD_END) return CD;
   if (resp != CMD_ACK)
    {
     printf ("Error receiving GC's answer!\n");
     return NULL;
    }

   temp = (card_dir *) malloc (sizeof(card_dir));
   if ( (ViperRead (temp, CARD_DIR_TRANSMIT_SIZE, &len) != 0) ||
        (len != CARD_DIR_TRANSMIT_SIZE))
    {
     free (temp);
     return CD;
    }

   temp->prox = NULL;
   *pos = temp;
   pos = &temp->prox;
  }

 return NULL;
}

// List the contents of memory card 1
void CMD_ListDir_MemCard_1 ()
{
 card_dir *CD;

 CD = CarregaCardDir (CMD_LISTDIR_MEMCARD_1); // Fetch list from memcard 1
 if (CD == NULL) return;
 PercorreLista (CD, ImprimeCardDir); // Prints it
 PercorreLista (CD, LivraCardDir); // Frees it
}

// List the contents of memory card 2
void CMD_ListDir_MemCard_2 ()
{
 card_dir *CD;

 CD = CarregaCardDir (CMD_LISTDIR_MEMCARD_2);
 if (CD == NULL) return;
 PercorreLista (CD, ImprimeCardDir);
 PercorreLista (CD, LivraCardDir);
}

void CMD_EraseFile ()
{
 unsigned char cmd;
 int channel;
 DWORD len;
 card_dir *CD, *temp;
 char buffer[80];

 if (PegaMemCard (&channel) == FALSE) return; // User, which mem card?
 CD = ListaDirMemCard (channel);  // Okay, here's it's contents
 if (CD == NULL) return;

 temp = PegaArqDaLista (CD);  // Which file you'd like to delete?
 if (temp == NULL) return;

 // Are you freaking sure? 
 printf ("Are you sure you'd like to delete the file:\n %s ??? ", temp->filename);
 printf ("Type 1337: ");  // Make it random later
 fgets (buffer, 79, stdin);
 
 if (atoi (buffer) == 1337)
   {
    SendCommand (CMD_ERASEFILE);   // Send info to the GC
    ViperWrite (temp, CARD_DIR_TRANSMIT_SIZE, &len);

    if ((GetCommand (&cmd) == FALSE) ||
        (cmd != CMD_ACK))
      printf ("Error erasing file!\n");
     else
      printf ("File erased successfully!\n");
   }
  else
   printf ("Operation cancelled.\n");

 PercorreLista (CD, LivraCardDir); // Free card_list dir
}

void CMD_Dumpfile ()
{
 DWORD TamTotal, TamBloco, TamPedaco;
 DWORD CheckSum1, CheckSum2, len;
 card_dir *CD, *temp;
 struct card_direntry status;

 int channel, arqnum, i;
 unsigned char *buffer, cmd;
 FILE *fout;

 if (PegaMemCard (&channel) == FALSE) return;
 CD = ListaDirMemCard (channel);
 if (CD == NULL) return;
 
 temp = PegaArqDaLista (CD);
 if (temp == NULL) return;

 printf ("DUMP : %s - %s - %s - %08X\n", temp->filename, temp->gamecode,
                                         temp->company, temp->chn);

 SendCommand (CMD_DUMPFILE);
 len = CARD_DIR_TRANSMIT_SIZE;
 ViperWrite (temp, CARD_DIR_TRANSMIT_SIZE, &len);

 if ((GetCommand (&cmd) == FALSE) ||
     (cmd != CMD_ACK))
  {
   printf ("Error retriving file from the GC!\n");
   PercorreLista (CD, LivraCardDir);
   return;
  }

 buffer = malloc (strlen (temp->filename) + 5); // .gci/0
 strcpy (buffer, temp->filename);
 strcat (buffer, ".gci");
 fout = fopen (buffer, "wb+");
 free (buffer);

 if (fout == NULL)
  {
   printf ("Error opening file '%s'\n", temp->filename);
   return;
  }

 ViperRead (&status, CARD_STAT_TRANSMIT_SIZE, &len);
 if (len != CARD_STAT_TRANSMIT_SIZE)
  {
   printf ("Protocol error!!");
   fclose (fout);
   return;
  }
 fwrite (&status, 1, CARD_STAT_TRANSMIT_SIZE, fout);

 TamTotal = GetDWORD ();
 TamBloco = GetDWORD ();
// printf ("TOTAL = %d, BLOCO = %d\n", TamTotal, TamBloco);
 printf ("Receiving %i blocks ", (int)(TamTotal/TamBloco));
 buffer = (unsigned char *) malloc (TamBloco);

 i = 1;
 while (1)
  {
   if (GetCommand (&cmd) == FALSE)
    { 
     printf ("Transmission error!\n");
     break;
    }

   if (cmd == CMD_ERR)
    {
     printf ("Error in the GC!\n");
     break;
    }

   if (cmd == CMD_END)
    {
     printf ("File transfered successfully!\n");
     break;
    }

   if (cmd == CMD_ACK) // Let's rock
    {
     TamPedaco = GetDWORD();
     //printf ("Pedaco = %d\n", TamPedaco);
     len = TamPedaco;
     ViperRead (buffer, TamPedaco, &len);
     //printf ("len = %d\n", len);
     
     if (len != TamPedaco)
      {
       printf ("Wrong-sized block!\n");
       break;
      }
     
     CheckSum1 = CalculaCheckSum (buffer, TamPedaco);
     CheckSum2 = GetDWORD ();

     if (CheckSum1 != CheckSum2)
       {
        //printf ("Erro de transmissao - tentando novamente!\n");
        printf ("x");
        fflush (stdout);
        SendCommand (CMD_ERR);
        continue;
       }
      else
       {
        printf (".");
        fflush (stdout);
        //printf ("Checksum bloco %d = %04X ok!\n", i, CheckSum1);
        i++;
       }
     
     // Se chegou aqui, beleza,  s gravar!
     SendCommand (CMD_ACK);
     fwrite (buffer, 1, TamPedaco, fout);
    }
  }

 free (buffer);
 fclose (fout);
 PercorreLista (CD, LivraCardDir);
}

void CMD_Upload_Proc (unsigned char UsrCmd)
{
 char *buffer, *bp;
 unsigned char cmd;
 int num_arqs, i, j, tam, channel;
 struct Lista *L, *temp, **pos;
 FILE *f = NULL;
 DIR *dp;
 struct dirent *ep;
 DWORD bsize, len, checksum;
 
 L = NULL;
 pos = &L;
 dp = opendir ("./");
 num_arqs = 0;
 if (dp != NULL)
   {
    while (ep = readdir (dp))
     {
      if (strstr (ep->d_name, ".gci") != NULL)
       {
        num_arqs++;
        printf ("[%d] - %s\n", num_arqs, ep->d_name);
        temp = malloc (sizeof (struct Lista));
        temp->arquivo = strdup (ep->d_name);
        temp->prox = NULL;
        *pos = temp;
        pos = &temp->prox;
       }
     }
    (void) closedir (dp);
   }
  else
   {
    printf ("Unable to open current directory!\n");
    return;
   }

 if (L == NULL)
  {
   printf("There are no GCI files in the current directory!\n");
   return;
  }

 do
  {
   printf ("Choose: ");
   if ((PegaInt (&i) == FALSE) ||
       (i < 0) ||
       (i > num_arqs))
     break;

   temp = L;
   j = 1;

   while (temp != NULL)
    {
     if (i == j) break;
     temp = temp->prox;
     j++;
    }
   if (temp == NULL) break;

   f = fopen (temp->arquivo, "rb");
   if (f == NULL) break;

   fseek (f, 0, SEEK_END);
   tam = ftell(f);
   fseek (f, 0, SEEK_SET);
   buffer = malloc (tam);

   //printf ("Abrindo arquivo...");
   if (fread (buffer, 1, tam, f) != tam) break;
   //printf ("OK\n");

   if (PegaMemCard (&channel) == FALSE) break;
   SendCommand (UsrCmd);
   SendCommand ((unsigned char) channel);

   if ((GetCommand(&cmd) == FALSE) || (cmd != CMD_ACK))
    {
     printf ("The GC is not ready to receive the file!\n");
     break;
    }
   bsize = GetDWORD ();
//   printf ("Bloco = %d\n", bsize);

  // printf ("Enviando CARD_STAT %d bytes...", CARD_STAT_TRANSMIT_SIZE);
   len = CARD_STAT_TRANSMIT_SIZE;
   ViperWrite (buffer, CARD_STAT_TRANSMIT_SIZE, &len);

   if ((GetCommand(&cmd) == FALSE) || (cmd != CMD_ACK)) break;
  // printf ("OK\n");

   bp = &buffer[CARD_STAT_TRANSMIT_SIZE];
   j = (tam - CARD_STAT_TRANSMIT_SIZE) / bsize;

//   printf ("tam = %d\n", tam);
//   printf ("bsize = %d\n", bsize);
//   printf ("tam - CSTS = %d\n", tam - CARD_STAT_TRANSMIT_SIZE);
//   printf ("j = %d\n", j);

   printf ("Sending %d blocks: ", j);
   for (i = 0; i < j; )
    {
     //printf ("Enviando bloco %d...", i+1);
     ViperWrite (bp, bsize, &len);
     checksum = CalculaCheckSum (bp, bsize);
     //printf (" CHK %04X...", checksum);
     PutDWORD (checksum);
     if (GetCommand(&cmd) == FALSE)
      {
       printf ("Timeout waiting the GC!\n");
       continue;
      }

     switch (cmd)
      {
       case CMD_ACK:
        printf (".");
        fflush (stdout);
        break;
       case CMD_END:
        printf ("\nFile sent successfully!\n");
        break;
       default:
        printf ("x");
        fflush(stdout);
        continue;
      }
 
     bp += bsize;
     i++;
    }

  } while (0);

 // Limpo a casa
 SendCommand (CMD_END);

 if (f != NULL) fclose (f);

 temp = L;
 while (temp != NULL)
  {
   pos = (void *) temp->prox;
   free (temp->arquivo);
   free (temp);
   temp = (void *) pos;
  }
}

void CMD_Checkfile ()
{
 CMD_Upload_Proc ((unsigned char)CMD_CHECKDUMP);
}

void CMD_Upload ()
{
 CMD_Upload_Proc ((unsigned char)CMD_UPLOAD);
}

void CMD_End ()
{
 exit (1);
}

struct Comando Lista[] = {
   {"List the contents of memory card 1", CMD_ListDir_MemCard_1},
   {"List the contents of memory card 2", CMD_ListDir_MemCard_2},
   {"Save file (GC->PC)", CMD_Dumpfile }, 
   {"Check file (PC->GC)", CMD_Checkfile },
   {"Upload (PC->GC)", CMD_Upload }, 
   {"Erase file (GC)", CMD_EraseFile }, 
   {"Exit", CMD_End} };

int main(int argc, char *argv[])
{
 int i, nmenu, item;
 DWORD len;
 char buffer[81];
 void (*func)(void);

 ViperOpen();
 /// Flush input buffers/cache
 len = 0;
 ViperSetReadTimeout(1);
 do 
  {
   ViperRead (&buffer[0], 32, &len);
  } while (len != 0);

 ViperSetReadTimeout(1000);
 ViperSetWriteTimeout(0);

 nmenu = sizeof (Lista) / sizeof (struct Comando);
 while (1)
  {
   for (i = 0; i < nmenu; i++)
     printf ("[%d] - %s\n", i+1, Lista[i].texto);

   printf ("Choose operation: ");
   buffer[0] = 0;
   fgets (buffer, 80, stdin);
   
   len = 1;
   item = atoi (buffer);
   if ((item < 1) || (item > nmenu)) continue;

   func = Lista[item-1].func;
   func();    
   
   printf ("\n");
  }

 return 0;
}
